@@ -1,4 +1,4 @@
-__version__ = "1.0.53"
+__version__ = "1.0.55"
 
 import kappadata.caching
 import kappadata.copying
@@ -8,18 +8,30 @@ import kappadata.transforms
 import kappadata.wrappers
 import kappadata.wrappers.dataset_wrappers
 import kappadata.wrappers.sample_wrappers
-from .caching.redis_dataset import RedisDataset
+from .batch_samplers.infinite_batch_sampler import InfiniteBatchSampler
 from .caching.shared_dict_dataset import SharedDictDataset
-from .collators.base.kd_collator import KDCollator
+from .collators.base.kd_single_collator import KDSingleCollator
 from .collators.base.kd_compose_collator import KDComposeCollator
 from .collators.pad_sequences_collator import PadSequencesCollator
-from .copying import copy_folder_from_global_to_local
+from .copying import copy_imagefolder_from_global_to_local
 from .datasets.kd_concat_dataset import KDConcatDataset
 from .datasets.kd_dataset import KDDataset
 from .datasets.kd_subset import KDSubset
 from .datasets.kd_wrapper import KDWrapper
 from .transforms.base.kd_compose_transform import KDComposeTransform
 from .transforms.base.kd_transform import KDTransform
+from .transforms.kd_color_jitter import KDColorJitter
+from .transforms.kd_gaussian_blur_pil import KDGaussianBlurPIL
+from .transforms.kd_gaussian_blur_tv import KDGaussianBlurTV
+from .transforms.kd_rand_augment import KDRandAugment
+from .transforms.kd_random_color_jitter import KDRandomColorJitter
+from .transforms.kd_random_erasing import KDRandomErasing
+from .transforms.kd_random_gaussian_blur_pil import KDRandomGaussianBlurPIL
+from .transforms.kd_random_gaussian_blur_tv import KDRandomGaussianBlurTV
+from .transforms.kd_random_grayscale import KDRandomGrayscale
+from .transforms.kd_random_horizontal_flip import KDRandomHorizontalFlip
+from .transforms.kd_random_resized_crop import KDRandomResizedCrop
+from .transforms.kd_random_solarize import KDRandomSolarize
 from .wrappers.dataset_wrappers.class_filter_wrapper import ClassFilterWrapper
 from .wrappers.dataset_wrappers.oversampling_wrapper import OversamplingWrapper
 from .wrappers.dataset_wrappers.percent_filter_wrapper import PercentFilterWrapper
